@@ -2,8 +2,10 @@
 
 import os
 import itertools
+
 from pysftpserver.abstractstorage import SFTPAbstractServerStorage
 from pysftpserver.futimes import futimes
+from pysftpserver.stat_helpers import stat_to_longname
 
 
 class SFTPServerStorage(SFTPAbstractServerStorage):
@@ -45,18 +47,35 @@ class SFTPServerStorage(SFTPAbstractServerStorage):
         elif lstat:
             _stat = os.lstat(filename)
         else:
-            _stat = os.stat(
-                filename if not parent
-                else os.path.join(parent, filename)
+            try:
+                _stat = os.stat(
+                    filename if not parent
+                    else os.path.join(parent, filename)
+                )
+            except:
+                # we could have a broken symlink
+                # but lstat could be false:
+                # this happens in case of readdir responses
+                _stat = os.lstat(
+                    filename if not parent
+                    else os.path.join(parent, filename)
+                )
+
+        if fstat:
+            longname = None  # not needed in case of fstat
+        else:
+            longname = stat_to_longname(  # see stat_helpers.py
+                _stat, filename
             )
 
         return {
-            'size': _stat.st_size,
-            'uid': _stat.st_uid,
-            'gid': _stat.st_gid,
-            'perm': _stat.st_mode,
-            'atime': _stat.st_atime,
-            'mtime': _stat.st_mtime,
+            b'size': _stat.st_size,
+            b'uid': _stat.st_uid,
+            b'gid': _stat.st_gid,
+            b'perm': _stat.st_mode,
+            b'atime': _stat.st_atime,
+            b'mtime': _stat.st_mtime,
+            b'longname': longname
         }
 
     def setstat(self, filename, attrs, fsetstat=False):
@@ -75,18 +94,18 @@ class SFTPServerStorage(SFTPAbstractServerStorage):
             chown = os.fchown
             chmod = os.fchmod
 
-        if 'size' in attrs:
-            os.ftruncate(f, attrs['size'])
-        if all(k in attrs for k in ('uid', 'gid')):
-            chown(filename, attrs['uid'], attrs['gid'])
-        if 'perm' in attrs:
-            chmod(filename, attrs['perm'])
+        if b'size' in attrs:
+            os.ftruncate(f, attrs[b'size'])
+        if all(k in attrs for k in (b'uid', b'gid')):
+            chown(filename, attrs[b'uid'], attrs[b'gid'])
+        if b'perm' in attrs:
+            chmod(filename, attrs[b'perm'])
 
-        if all(k in attrs for k in ('atime', 'mtime')):
+        if all(k in attrs for k in (b'atime', b'mtime')):
             if not fsetstat:
-                os.utime(filename, (attrs['atime'], attrs['mtime']))
+                os.utime(filename, (attrs[b'atime'], attrs[b'mtime']))
             else:
-                futimes(filename, (attrs['atime'], attrs['mtime']))
+                futimes(filename, (attrs[b'atime'], attrs[b'mtime']))
 
     def opendir(self, filename):
         """Return an iterator over the files in filename."""
